require 'ball'
require 'player'
require 'chipmunk'

#DTIME is basically how long each substep
# is.  Lower values will increase the accuracy while higher help performance.
SUBSTEPS, DTIME = 3, (1.0 / 30.0)
SCREEN_WIDTH, SCREEN_HEIGHT = 1150, 800

class Game < Gosu::Window
  def initialize
    super(SCREEN_WIDTH,SCREEN_HEIGHT,false)
    self.caption = "Footy!"
    setup_scene
  end

  def setup_scene
    # Setup all images
    @ball_image = Gosu::Image.new(self, "graphics/ball8.png", true)
    @player_image = Gosu::Image.new(self, "graphics/smile.png", true)
    @background_image = Gosu::Image.new(self, "graphics/pitch.png", true)

    # Players
    @players = Array.new
    
    # Setup our space with resistance
    @space = CP::Space.new
    @space.damping = 0.001
    @space.gravity = CP::Vec2.new(0.0, 0.0)
    @space.elastic_iterations = 3

    @static_body = CP::Body.new((1.0/0.0), (1.0/0.0))

    # borders
    draw_structure( 0.0, 0.0, 0.0, SCREEN_WIDTH, :wall )
    draw_structure( SCREEN_WIDTH, 0.0, SCREEN_WIDTH, SCREEN_HEIGHT, :wall )
    draw_structure( 0.0, SCREEN_HEIGHT, SCREEN_WIDTH, SCREEN_HEIGHT, :wall )

    @ball = Ball.new(@space, @ball_image)
    add_player
    @current_player = @players.first
  end

  def draw_structure(x_from,y_from,x_to,y_to,group)
    sp=CP::Shape::Segment.new(@static_body, CP::Vec2.new(x_from,   y_from), CP::Vec2.new(x_to, y_to), 1.0)
    sp.e = 0.9 #elasticity
    sp.u = 0.1 #friction
    sp.collision_type=group
    @space.add_static_shape sp
  end

  def add_player
    player = Player.new(@space, @player_image)

    @players.push(player)
  end

  def update
    # Perform physics steps first
    SUBSTEPS.times do
      @space.step(DTIME)
      @current_player.body.reset_forces
      process_keyboard_inputs
    end


  end

  def process_keyboard_inputs
    @current_player.body.reset_forces
    @current_player.move_left if button_down? Gosu::Button::KbLeft
    @current_player.move_right if button_down? Gosu::Button::KbRight
    @current_player.move_up if button_down? Gosu::Button::KbUp
    @current_player.move_down if button_down? Gosu::Button::KbDown
  end

  def button_up(id)
    SUBSTEPS.times do
      @space.step(DTIME)
      @current_player.body.reset_forces
      @current_player.move_right if id == Gosu::Button::KbLeft
    end

  end

  def draw
    @background_image.draw(0,0,0)
    @ball.draw
    @players.each do |player|
      player.draw
    end
  end
  
end